/*global module Float32Array Float64Array*/
(function(){"use strict";
	function Matrix3 (mat) {return this.initMatrix3(mat);}
	var matrix3 = Matrix3.prototype;
	Matrix3.check = function (obj) {
		return obj && obj.initMatrix3 === matrix3.initMatrix3;
	};
	Matrix3.create = function (obj, args) {
		obj = obj || [];
		obj.length = 9;
		obj.initMatrix3 = matrix3.initMatrix3 || function (mat) {
			mat = mat || [];
			this[0] = mat[0] || 0;
			this[1] = mat[1] || 0;
			this[2] = mat[2] || 0;
			this[3] = mat[3] || 0;
			this[4] = mat[4] || 0;
			this[5] = mat[5] || 0;
			this[6] = mat[6] || 0;
			this[7] = mat[7] || 0;
			this[8] = mat[8] || 0;
			return this;
		};
		obj.set = function (mat) {
			return this.initMatrix3(mat);
		};
		obj.copy = function () {
			return new Matrix3(this);
		};
		obj.identity = function () {
			return this.set([
				1, 0, 0,
				0, 1, 0,
				0, 0, 1
			]);
		};
		obj.transpose = function (mat) {
			var a01, a02, a12;
			if (!mat) {
				a01 = this[1];
				a02 = this[2];
				a12 = this[5];
				this[1] = this[3];
				this[2] = this[6];
				this[3] = a01;
				this[5] = this[7];
				this[6] = a02;
				this[7] = a12;
			} else {
				this[0] = mat[0];
				this[1] = mat[3];
				this[2] = mat[6];
				this[3] = mat[1];
				this[4] = mat[4];
				this[5] = mat[7];
				this[6] = mat[2];
				this[7] = mat[5];
				this[8] = mat[8];
			}
			return this;
		};
		obj.forEach = function (fn, bind) {
			if (typeof fn !== 'function') return this;
			bind = bind || this;
			for (var i = 0, l = this.length; i < l; i += 1) {
				fn.call(bind, this[i], i, this);
			}
			return this;
		};
		obj.toMat4 = function () {
			return [
				this[0], this[1], this[2], 0,
				this[3], this[4], this[5], 0,
				this[6], this[7], this[8], 0,
				      0,       0,       0, 1
			];
		};
		obj.toString = obj.str = function () {
			return this.toArray().join(',');
		};
		obj.toArray = function () {
			return [
				this[0], this[1], this[2],
				this[3], this[4], this[5],
				this[6], this[7], this[8]
			];
		};
		obj.toFloat32Array = function () {
			return new Float32Array(this.toArray());
		};
		obj.toFloat64Array = function () {
			return new Float64Array(this.toArray());
		};
		if (args) obj.initMatrix3.apply(obj, args);
		return obj;
	};
	(module.exports = Matrix3).create(matrix3);
}).call(this);