/*global module require*/
(function(){"use strict";
	function Matrix4 (mat) {return this.initMatrix4(mat);}
	var Matrix3 = require('./Matrix3'),
	    matrix4 = Matrix4.prototype;
	Matrix4.check = function (obj) {
		return Matrix3.check(obj) && obj.initMatrix4 === matrix4.initMatrix4;
	};
	Matrix4.create = function (obj, args) {
		obj = obj || [];
		if (!Matrix3.check(obj)) Matrix3.create(obj);
		obj.length = 16;
		obj.initMatrix4 = matrix4.initMatrix4 || function (mat) {
			mat = mat || [];
			this[ 0] = mat[ 0] || 0;
			this[ 1] = mat[ 1] || 0;
			this[ 2] = mat[ 2] || 0;
			this[ 3] = mat[ 3] || 0;
			this[ 4] = mat[ 4] || 0;
			this[ 5] = mat[ 5] || 0;
			this[ 6] = mat[ 6] || 0;
			this[ 7] = mat[ 7] || 0;
			this[ 8] = mat[ 8] || 0;
			this[ 9] = mat[ 9] || 0;
			this[10] = mat[10] || 0;
			this[11] = mat[11] || 0;
			this[12] = mat[12] || 0;
			this[13] = mat[13] || 0;
			this[14] = mat[14] || 0;
			this[15] = mat[15] || 0;
			return this;
		};
		obj.set = function (mat) {
			return this.initMatrix4(mat);
		};
		obj.copy = function () {
			return new Matrix4(this);
		};
		obj.identity = function () {
			return this.set([
				1, 0, 0, 0,
				0, 1, 0, 0,
				0, 0, 1, 0,
				0, 0, 0, 1
			]);
		};
		obj.transpose = function (mat) {
			var a01, a02, a03, a12, a13, a23;
			if (!mat) {
				a01 = this[ 1];
				a02 = this[ 2];
				a03 = this[ 3];
				a12 = this[ 6];
				a13 = this[ 7];
				a23 = this[11];
				this[1] = this[4];
				this[2] = this[8];
				this[3] = this[12];
				this[4] = a01;
				this[6] = this[9];
				this[7] = this[13];
				this[8] = a02;
				this[9] = a12;
				this[11] = this[14];
				this[12] = a03;
				this[13] = a13;
				this[14] = a23;
			} else {
				this[0] = mat[0];
				this[1] = mat[4];
				this[2] = mat[8];
				this[3] = mat[12];
				this[4] = mat[1];
				this[5] = mat[5];
				this[6] = mat[9];
				this[7] = mat[13];
				this[8] = mat[2];
				this[9] = mat[6];
				this[10] = mat[10];
				this[11] = mat[14];
				this[12] = mat[3];
				this[13] = mat[7];
				this[14] = mat[11];
				this[15] = mat[15];
			}
			return this;
		};
		obj.determinant = function () {
			var a00 = this[ 0], a01 = this[ 1], a02 = this[ 2], a03 = this[ 3],
			    a10 = this[ 4], a11 = this[ 5], a12 = this[ 6], a13 = this[ 7],
			    a20 = this[ 8], a21 = this[ 9], a22 = this[10], a23 = this[11],
			    a30 = this[12], a31 = this[13], a32 = this[14], a33 = this[15];
			return a30*a21*a12*a03 - a20*a31*a12*a03 - a30*a11*a22*a03 + a10*a31*a22*a03 +
			       a20*a11*a32*a03 - a10*a21*a32*a03 - a30*a21*a02*a13 + a20*a31*a02*a13 +
			       a30*a01*a22*a13 - a00*a31*a22*a13 - a20*a01*a32*a13 + a00*a21*a32*a13 +
			       a30*a11*a02*a23 - a10*a31*a02*a23 - a30*a01*a12*a23 + a00*a31*a12*a23 +
			       a10*a01*a32*a23 - a00*a11*a32*a23 - a20*a11*a02*a33 + a10*a21*a02*a33 +
			       a20*a01*a12*a33 - a00*a21*a12*a33 - a10*a01*a22*a33 + a00*a11*a22*a33;
		};
		obj.inverse = function () {
			var a00 = this[ 0], a01 = this[ 1], a02 = this[ 2], a03 = this[ 3],
			    a10 = this[ 4], a11 = this[ 5], a12 = this[ 6], a13 = this[ 7],
			    a20 = this[ 8], a21 = this[ 9], a22 = this[10], a23 = this[11],
			    a30 = this[12], a31 = this[13], a32 = this[14], a33 = this[15],
			    b00 = a00*a11 - a01*a10,
			    b01 = a00*a12 - a02*a10,
			    b02 = a00*a13 - a03*a10,
			    b03 = a01*a12 - a02*a11,
			    b04 = a01*a13 - a03*a11,
			    b05 = a02*a13 - a03*a12,
			    b06 = a20*a31 - a21*a30,
			    b07 = a20*a32 - a22*a30,
			    b08 = a20*a33 - a23*a30,
			    b09 = a21*a32 - a22*a31,
			    b10 = a21*a33 - a23*a31,
			    b11 = a22*a33 - a23*a32,
			    invDet = 1/(b00*b11 - b01*b10 + b02*b09 + b03*b08 - b04*b07 + b05*b06);
			this[ 0] = ( a11*b11 - a12*b10 + a13*b09)*invDet;
			this[ 1] = (-a01*b11 + a02*b10 - a03*b09)*invDet;
			this[ 2] = ( a31*b05 - a32*b04 + a33*b03)*invDet;
			this[ 3] = (-a21*b05 + a22*b04 - a23*b03)*invDet;
			this[ 4] = (-a10*b11 + a12*b08 - a13*b07)*invDet;
			this[ 5] = ( a00*b11 - a02*b08 + a03*b07)*invDet;
			this[ 6] = (-a30*b05 + a32*b02 - a33*b01)*invDet;
			this[ 7] = ( a20*b05 - a22*b02 + a23*b01)*invDet;
			this[ 8] = ( a10*b10 - a11*b08 + a13*b06)*invDet;
			this[ 9] = (-a00*b10 + a01*b08 - a03*b06)*invDet;
			this[10] = ( a30*b04 - a31*b02 + a33*b00)*invDet;
			this[11] = (-a20*b04 + a21*b02 - a23*b00)*invDet;
			this[12] = (-a10*b09 + a11*b07 - a12*b06)*invDet;
			this[13] = ( a00*b09 - a01*b07 + a02*b06)*invDet;
			this[14] = (-a30*b03 + a31*b01 - a32*b00)*invDet;
			this[15] = ( a20*b03 - a21*b01 + a22*b00)*invDet;
			return this;
		};
		obj.toRotationMat = function () {
			return [
				this[0], this[1], this[ 2], this[ 3],
				this[4], this[5], this[ 6], this[ 7],
				this[8], this[9], this[10], this[11],
				      0,       0,        0,        1
			];
		};
		obj.toMat3 = function () {
			return [
				this[0], this[1], this[ 2],
				this[4], this[5], this[ 6],
				this[8], this[9], this[10]
			];
		};
		obj.toInverseMat3 = function () {
			var a00 = this[0], a01 = this[1], a02 = this[ 2],
			    a10 = this[4], a11 = this[5], a12 = this[ 6],
			    a20 = this[8], a21 = this[9], a22 = this[10],
			    b01 =  a22*a11 - a12*a21,
			    b11 = -a22*a10 + a12*a20,
			    b21 =  a21*a10 - a11*a20,
			    d = a00*b01 + a01*b11 + a02*b21,
			    id;
			if (!d) return null;
			id = 1/d;
			return [
				b01*id,
				(-a22*a01 + a02*a21)*id,
				(a12*a01 - a02*a11)*id,
				b11*id,
				(a22*a00 - a02*a20)*id,
				(-a12*a00 + a02*a10)*id,
				b21*id,
				(-a21*a00 + a01*a20)*id,
				(a11*a00 - a01*a10)*id
			];
		};
		obj.multiply = function (mat) {
			var a00 = this[ 0], a01 = this[ 1], a02 = this[ 2], a03 = this[ 3],
			    a10 = this[ 4], a11 = this[ 5], a12 = this[ 6], a13 = this[ 7],
			    a20 = this[ 8], a21 = this[ 9], a22 = this[10], a23 = this[11],
			    a30 = this[12], a31 = this[13], a32 = this[14], a33 = this[15],
			    b00 =  mat[ 0], b01 =  mat[ 1], b02 =  mat[ 2], b03 =  mat[ 3],
			    b10 =  mat[ 4], b11 =  mat[ 5], b12 =  mat[ 6], b13 =  mat[ 7],
			    b20 =  mat[ 8], b21 =  mat[ 9], b22 =  mat[10], b23 =  mat[11],
			    b30 =  mat[12], b31 =  mat[13], b32 =  mat[14], b33 =  mat[15];
			this[0] = b00*a00 + b01*a10 + b02*a20 + b03*a30;
			this[1] = b00*a01 + b01*a11 + b02*a21 + b03*a31;
			this[2] = b00*a02 + b01*a12 + b02*a22 + b03*a32;
			this[3] = b00*a03 + b01*a13 + b02*a23 + b03*a33;
			this[4] = b10*a00 + b11*a10 + b12*a20 + b13*a30;
			this[5] = b10*a01 + b11*a11 + b12*a21 + b13*a31;
			this[6] = b10*a02 + b11*a12 + b12*a22 + b13*a32;
			this[7] = b10*a03 + b11*a13 + b12*a23 + b13*a33;
			this[8] = b20*a00 + b21*a10 + b22*a20 + b23*a30;
			this[9] = b20*a01 + b21*a11 + b22*a21 + b23*a31;
			this[10] = b20*a02 + b21*a12 + b22*a22 + b23*a32;
			this[11] = b20*a03 + b21*a13 + b22*a23 + b23*a33;
			this[12] = b30*a00 + b31*a10 + b32*a20 + b33*a30;
			this[13] = b30*a01 + b31*a11 + b32*a21 + b33*a31;
			this[14] = b30*a02 + b31*a12 + b32*a22 + b33*a32;
			this[15] = b30*a03 + b31*a13 + b32*a23 + b33*a33;
			return this;
		};
		obj.multiplyVec3 = function (vec) {
			var x = vec[0], y = vec[1], z = vec[2];
			vec[0] = this[0]*x + this[4]*y + this[ 8]*z + this[12];
			vec[1] = this[1]*x + this[5]*y + this[ 9]*z + this[13];
			vec[2] = this[2]*x + this[6]*y + this[10]*z + this[14];
			return this;
		};
		obj.multiplyVec4 = function (vec) {
			var x = vec[0], y = vec[1], z = vec[2], w = vec[3];
			vec[0] = this[0]*x + this[4]*y + this[ 8]*z + this[12]*w;
			vec[1] = this[1]*x + this[5]*y + this[ 9]*z + this[13]*w;
			vec[2] = this[2]*x + this[6]*y + this[10]*z + this[14]*w;
			vec[3] = this[3]*x + this[7]*y + this[11]*z + this[15]*w;
			return this;
		};
		obj.translate = function (vec) {
			var x = vec[0], y = vec[1], z = vec[2];
			this[12] = this[0]*x + this[4]*y + this[ 8]*z + this[12];
			this[13] = this[1]*x + this[5]*y + this[ 9]*z + this[13];
			this[14] = this[2]*x + this[6]*y + this[10]*z + this[14];
			this[15] = this[3]*x + this[7]*y + this[11]*z + this[15];
			return this;
		};
		obj.scale = function (vec) {
			var x = vec[0], y = vec[1], z = vec[2];
			this[ 0] *= x;
			this[ 1] *= x;
			this[ 2] *= x;
			this[ 3] *= x;
			this[ 4] *= y;
			this[ 5] *= y;
			this[ 6] *= y;
			this[ 7] *= y;
			this[ 8] *= z;
			this[ 9] *= z;
			this[10] *= z;
			this[11] *= z;
			return this;
		};
		obj.rotate = function (angle, axis) {
			var x = axis[0],
			    y = axis[1],
			    z = axis[2],
			    len = Math.sqrt(x*x + y*y + z*z);
			if (!len) return null;
			if (len !== 1) {
				len = 1 / len;
				x *= len; 
				y *= len; 
				z *= len;
			}
			var s = Math.sin(angle), c = Math.cos(angle), t = 1 - c,
			    a00 = this[0], a01 = this[1], a02 = this[ 2], a03 = this[ 3],
			    a10 = this[4], a11 = this[5], a12 = this[ 6], a13 = this[ 7],
			    a20 = this[8], a21 = this[9], a22 = this[10], a23 = this[11],
			    b00 = x*x*t + c,   b01 = y*x*t + z*s, b02 = z*x*t - y*s,
			    b10 = x*y*t - z*s, b11 = y*y*t + c,   b12 = z*y*t + x*s,
			    b20 = x*z*t + y*s, b21 = y*z*t - x*s, b22 = z*z*t + c;
			this[ 0] = a00*b00 + a10*b01 + a20*b02;
			this[ 1] = a01*b00 + a11*b01 + a21*b02;
			this[ 2] = a02*b00 + a12*b01 + a22*b02;
			this[ 3] = a03*b00 + a13*b01 + a23*b02;
			this[ 4] = a00*b10 + a10*b11 + a20*b12;
			this[ 5] = a01*b10 + a11*b11 + a21*b12;
			this[ 6] = a02*b10 + a12*b11 + a22*b12;
			this[ 7] = a03*b10 + a13*b11 + a23*b12;
			this[ 8] = a00*b20 + a10*b21 + a20*b22;
			this[ 9] = a01*b20 + a11*b21 + a21*b22;
			this[10] = a02*b20 + a12*b21 + a22*b22;
			this[11] = a03*b20 + a13*b21 + a23*b22;
			return this;
		};
		obj.rotateX = function (angle) {
			var s = Math.sin(angle), c = Math.cos(angle),
			    a10 = this[4], a11 = this[5], a12 = this[ 6], a13 = this[ 7],
			    a20 = this[8], a21 = this[9], a22 = this[10], a23 = this[11];
			this[ 4] = a10*c + a20*s;
			this[ 5] = a11*c + a21*s;
			this[ 6] = a12*c + a22*s;
			this[ 7] = a13*c + a23*s;
			this[ 8] = a10*-s + a20*c;
			this[ 9] = a11*-s + a21*c;
			this[10] = a12*-s + a22*c;
			this[11] = a13*-s + a23*c;
			return this;
		};
		obj.rotateY = function (angle) {
			var s = Math.sin(angle), c = Math.cos(angle),
			    a00 = this[0], a01 = this[1], a02 = this[ 2], a03 = this[ 3],
			    a20 = this[8], a21 = this[9], a22 = this[10], a23 = this[11];
			this[ 0] = a00*c + a20*-s;
			this[ 1] = a01*c + a21*-s;
			this[ 2] = a02*c + a22*-s;
			this[ 3] = a03*c + a23*-s;
			this[ 8] = a00*s + a20*c;
			this[ 9] = a01*s + a21*c;
			this[10] = a02*s + a22*c;
			this[11] = a03*s + a23*c;
			return this;
		};
		obj.rotateZ = function (angle) {
			var s = Math.sin(angle), c = Math.cos(angle),
			    a00 = this[0], a01 = this[1], a02 = this[2], a03 = this[3],
			    a10 = this[4], a11 = this[5], a12 = this[6], a13 = this[7];
			this[0] = a00*c + a10*s;
			this[1] = a01*c + a11*s;
			this[2] = a02*c + a12*s;
			this[3] = a03*c + a13*s;
			this[4] = a00*-s + a10*c;
			this[5] = a01*-s + a11*c;
			this[6] = a02*-s + a12*c;
			this[7] = a03*-s + a13*c;
			return this;
		};
		obj.frustum = function (left, right, bottom, top, near, far) {
			var rl = (right - left),
			    tb = (top - bottom),
			    fn = (far - near);
			this[ 0] = (near*2) / rl;
			this[ 1] = 0;
			this[ 2] = 0;
			this[ 3] = 0;
			this[ 4] = 0;
			this[ 5] = (near*2) / tb;
			this[ 6] = 0;
			this[ 7] = 0;
			this[ 8] = (right + left) / rl;
			this[ 9] = (top + bottom) / tb;
			this[10] = -(far + near) / fn;
			this[11] = -1;
			this[12] = 0;
			this[13] = 0;
			this[14] = -(far*near*2) / fn;
			this[15] = 0;
			return this;
		};
		obj.perspective = function (fovy, aspect, near, far) {
			var top = near*Math.tan(fovy*Math.PI / 360.0),
			    right = top*aspect;
			return this.frustum(-right, right, -top, top, near, far);
		};
		obj.ortho = function (left, right, bottom, top, near, far) {
			var rl = (right - left),
			    tb = (top - bottom),
			    fn = (far - near);
			this[ 0] = 2 / rl;
			this[ 1] = 0;
			this[ 2] = 0;
			this[ 3] = 0;
			this[ 4] = 0;
			this[ 5] = 2 / tb;
			this[ 6] = 0;
			this[ 7] = 0;
			this[ 8] = 0;
			this[ 9] = 0;
			this[10] = -2 / fn;
			this[11] = 0;
			this[12] = -(left + right) / rl;
			this[13] = -(top + bottom) / tb;
			this[14] = -(far + near) / fn;
			this[15] = 1;
			return this;
		};
		obj.lookAt = function (eye, target, up) {
			var eyex = eye[0],
			    eyey = eye[1],
			    eyez = eye[2],
			    upx = up[0],
			    upy = up[1],
			    upz = up[2],
			    targetx = target[0],
			    targety = target[1],
			    targetz = target[2],
			    z0, z1, z2,
			    x0, x1, x2,
			    y0, y1, y2,
			    len;
			if (eyex === targetx && eyey === targety && eyez === targetz) {
				return this.identity();
			}
			z0 = eyex - target[0];
			z1 = eyey - target[1];
			z2 = eyez - target[2];
			len = 1/Math.sqrt(z0*z0 + z1*z1 + z2*z2);
			z0 *= len;
			z1 *= len;
			z2 *= len;
			x0 = upy*z2 - upz*z1;
			x1 = upz*z0 - upx*z2;
			x2 = upx*z1 - upy*z0;
			len = Math.sqrt(x0*x0 + x1*x1 + x2*x2);
			if (!len) {
				x0 = 0;
				x1 = 0;
				x2 = 0;
			} else {
				len = 1/len;
				x0 *= len;
				x1 *= len;
				x2 *= len;
			}
			y0 = z1*x2 - z2*x1;
			y1 = z2*x0 - z0*x2;
			y2 = z0*x1 - z1*x0;
			len = Math.sqrt(y0*y0 + y1*y1 + y2*y2);
			if (!len) {
				y0 = 0;
				y1 = 0;
				y2 = 0;
			} else {
				len = 1/len;
				y0 *= len;
				y1 *= len;
				y2 *= len;
			}
			this[0] = x0;
			this[1] = y0;
			this[2] = z0;
			this[3] = 0;
			this[4] = x1;
			this[5] = y1;
			this[6] = z1;
			this[7] = 0;
			this[8] = x2;
			this[9] = y2;
			this[10] = z2;
			this[11] = 0;
			this[12] = -(x0*eyex + x1*eyey + x2*eyez);
			this[13] = -(y0*eyex + y1*eyey + y2*eyez);
			this[14] = -(z0*eyex + z1*eyey + z2*eyez);
			this[15] = 1;
			return this;
		};
		obj.toArray = function () {
			return [
				this[ 0], this[ 1], this[ 2], this[ 3],
				this[ 4], this[ 5], this[ 6], this[ 7],
				this[ 8], this[ 9], this[10], this[11],
				this[12], this[13], this[14], this[15]
			];
		};
		if (args) obj.initMatrix4.apply(obj, args);
		return obj;
	};
	(module.exports = Matrix4).create(matrix4);
}).call(this);