/*global module Float32Array Float64Array*/
(function(){"use strict";
	function Vector3 (vec) {return this.initVector3(vec);}
	var vector3 = Vector3.prototype;
	Vector3.check = function (obj) {
		return obj && obj.initVector3 === vector3.initVector3;
	};
	Vector3.create = function (obj, args) {
		obj = obj || [];
		obj.length = 3;
		obj.initVector3 = vector3.initVector3 || function (vec) {
			vec = vec || [];
			this[0] = vec[0] || 0;
			this[1] = vec[1] || 0;
			this[2] = vec[2] || 0;
			return this;
		};
		obj.set = function (vec) {
			return this.initVector3(vec);
		};
		obj.copy = function (vec) {
			return new Vector3(this);
		};
		obj.add = function (vec) {
			this[0] += vec[0];
			this[1] += vec[1];
			this[2] += vec[2];
			return this;
		};
		obj.subtract = function (vec) {
			this[0] -= vec[0];
			this[1] -= vec[1];
			this[2] -= vec[2];
			return this;
		};
		obj.negate = function (vec) {
			this[0] = -this[0];
			this[1] = -this[1];
			this[2] = -this[2];
			return this;
		};
		obj.multiply = function (vec) {
			this[0] *= vec[0];
			this[1] *= vec[1];
			this[2] *= vec[2];
			return this;
		};
		obj.divide = function (vec) {
			this[0] /= vec[0];
			this[1] /= vec[1];
			this[2] /= vec[2];
			return this;
		};
		obj.scale = function (val) {
			this[0] *= val;
			this[1] *= val;
			this[2] *= val;
			return this;
		};
		obj.zero = function () {
			this[0] = this[1] = this[2] = 0;
			return this;
		};
		obj.normalize = function () {
			var len = this.len();
			if (!len) this.zero();
			else if (len !== 1) {
				len = 1 / len;
				this[0] *= len;
				this[1] *= len;
				this[2] *= len;
			}
			return this;
		};
		obj.direction = function (vec) {
			var sub = this.copy().subtract(vec),
			    len = sub.len();
			if (!len) this.zero();
			else {
				len = 1 / len;
				this[0] = sub[0] * len;
				this[1] = sub[1] * len;
				this[2] = sub[2] * len;
			}
			return this;
		};
		obj.len = function () {
			return Math.sqrt(this.dot(this));
		};
		obj.dot = function (vec) {
			return this[0]*vec[0] + this[1]*vec[1] + this[2]*vec[2];
		};
		obj.cross = function (vec) {
			var  x = this[0], y = this[1], z = this[2],
			    x2 = vec[0], y2 = vec[1], z2 = vec[2];
			this[0] = y*z2 - z*y2;
			this[1] = z*x2 - x*z2;
			this[2] = x*y2 - y*x2;
			return this;
		};
		obj.lerp = function (vec, lerp) {
			var x = this[0], y = this[1], z = this[2];
			this[0] = x + lerp * (vec[0] - x);
			this[1] = y + lerp * (vec[1] - y);
			this[2] = z + lerp * (vec[2] - z);
			return this;
		};
		obj.toString = obj.str = function (vec) {
			return this.toArray().join(',');
		};
		obj.toArray = function () {
			return [this[0], this[1], this[2]];
		};
		obj.toFloat32Array = function () {
			return new Float32Array(this.toArray());
		};
		obj.toFloat64Array = function () {
			return new Float64Array(this.toArray());
		};
		return obj;
	};
	(module.exports = Vector3).create(vector3);
}).call(this);